--	=================================================================================================================================
--	Copyright 2012, Marcel Lambrechts <marcel.lambrechts@gmail.com>
--
--	This script is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation, either version 3 of the License, or
--	(at your option) any later version.
--
--	This script is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--
--	You should have received a copy of the GNU General Public License
--	along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	
--	Created: 20-05-2012
--	Version: 1.0
--
--	Decription:
--	This script will create a function named "mcl_genpwd" that can be used to generate random passwords.
--
--	 Function arguments:
--    p_length      - Length of the password that should be generated (Default: 10)
--    p_min_letters - The minimum number of letters that should be in the generated password (Default: 2)
--    p_min_digits  - The minimum number of digits (or special) characters that should be in the generated password (Default: 2)
--    p_mixed_case  - Shoud the generated password contain mixed case characters (Default: YES)
--
--   Function return value:
--    This function will return a random password as a VARCHAR2
--	=================================================================================================================================

CREATE OR REPLACE FUNCTION mcl_genpwd(p_length IN NUMBER DEFAULT 10, p_min_letters IN NUMBER DEFAULT 2, p_min_digits IN NUMBER DEFAULT 2, p_mixed_case IN BOOLEAN DEFAULT TRUE) RETURN VARCHAR2 IS
  c_digit_array     VARCHAR2(13) := '0123456789#$_';
  c_letter_array    VARCHAR2(52) := 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';

  l_letters         NUMBER;
  l_digits          NUMBER;
  l_password        VARCHAR2(30);
  l_tmp_password    VARCHAR2(30);
  l_todo            NUMBER;
  l_type            NUMBER;
BEGIN
  IF (p_length < p_min_letters + p_min_digits) THEN
    raise_application_error(-20002, 'The sum of the minimum number of digits and letters cannot be larger than the password length');
  END IF;
  
  -- Password MUST start with a letter (Oracle requirement)
  l_password := substr(c_letter_array, dbms_random.value(1, length(c_letter_array)), 1);
  
  -- Get a character, one at the time based on a random values
  l_letters := 1;
  l_digits := 0;

  FOR i IN 2..p_length LOOP
    l_todo := p_length - (i - 1);
    l_type := ROUND(dbms_random.value(1,10),0);
	
    IF (l_type IN (1,2,3,4)) THEN
        IF (l_todo > p_min_letters - l_letters) THEN 
          l_password := l_password||substr(c_digit_array, round(dbms_random.value(1, length(c_digit_array)),0), 1);
          l_digits := l_digits + 1;
        ELSE
          l_password := l_password||substr(c_letter_array, round(dbms_random.value(1, length(c_letter_array)),0), 1);
          l_letters := l_letters + 1;
        END IF;
    ELSE
       IF (l_todo > p_min_digits - l_digits) THEN 
          l_password := l_password||substr(c_letter_array, round(dbms_random.value(1, length(c_letter_array)),0), 1);
          l_letters := l_letters + 1;
        ELSE
          l_password := l_password||substr(c_digit_array, round(dbms_random.value(1, length(c_digit_array)),0), 1);
          l_digits := l_digits + 1;
        END IF;
	END IF;
  END LOOP;
  
  -- Make sure we generated a mixed case password if required
  IF (p_mixed_case AND (l_password = lower(l_password) OR l_password = upper(l_password))) THEN
    l_tmp_password := lower(substr(l_password, 1, 1)) || substr(l_password, 2, length(l_password) - 1);
    IF (l_password != l_tmp_password) THEN
      l_password := l_tmp_password;
    ELSE
      l_password := upper(substr(l_password, 1, 1)) || substr(l_password, 2, length(l_password) - 1);
    END IF;
  END IF;
  
  RETURN l_password;
END;
/